@@ -12,22 +12,7 @@
 #include <unistd.h>
 #include <sys/mman.h>
 
-#ifdef HAVE_ALLOCA_H
-# include <alloca.h>
-#elif defined __GNUC__
-# define alloca __builtin_alloca
-#elif defined _AIX
-# define alloca __alloca
-#elif defined _MSC_VER
-# include <malloc.h>
-# define alloca _alloca
-#else
-# include <stddef.h>
-# ifdef  __cplusplus
-extern "C"
-# endif
-void *alloca (size_t);
-#endif
+#include <Ecore_File.h>
 
 #include "Efreet.h"
 #include "efreet_private.h"
@@ -39,8 +24,7 @@ void *alloca (size_t);
 static int _efreet_ini_log_dom = -1;
 
 static Eina_Hash *efreet_ini_parse(const char *file);
-static char *efreet_ini_unescape(const char *str);
-static int _efreet_ini_log_om = -1;
+static const char *efreet_ini_unescape(const char *str);
 static Eina_Bool
 efreet_ini_section_save(const Eina_Hash *hash, const void *key, void *data, void *fdata);
 static Eina_Bool
@@ -57,8 +41,8 @@ efreet_ini_init(void)
     _efreet_ini_log_dom = eina_log_domain_register("Efreet_init", EFREET_DEFAULT_LOG_COLOR);
     if (_efreet_ini_log_dom < 0)
     {
-	ERROR("Efreet: Could not create a log domain for efreet_init");
-	return 0;
+        ERROR("Efreet: Could not create a log domain for efreet_init");
+        return 0;
     }
     return 1;
 }
@@ -184,9 +168,9 @@ efreet_ini_parse(const char *file)
                 memcpy((char*)header, line_start + 1, header_length - 1);
                 ((char*)header)[header_length - 1] = '\0';
 
-                section = eina_hash_string_small_new(free);
+                section = eina_hash_string_small_new(EINA_FREE_CB(eina_stringshare_del));
 
-                eina_hash_del(data, header, NULL);
+                eina_hash_del_by_key(data, header);
 //                if (old) INF("[efreet] Warning: duplicate section '%s' "
   //                              "in file '%s'", header, file);
 
@@ -261,7 +245,7 @@ efreet_ini_parse(const char *file)
                     value_end - value_start);
             ((char*)value)[value_end - value_start] = '\0';
 
-            eina_hash_del(section, key, NULL);
+            eina_hash_del_by_key(section, key);
             eina_hash_add(section, key, efreet_ini_unescape(value));
         }
 //        else
@@ -277,6 +261,13 @@ next_line:
     munmap((char*) buffer, file_stat.st_size);
     fclose(f);
 
+#if 0
+    if (!eina_hash_population(data))
+    {
+        eina_hash_free(data);
+        return NULL;
+    }
+#endif
     return data;
 }
 
@@ -303,9 +294,17 @@ efreet_ini_free(Efreet_Ini *ini)
 EAPI int
 efreet_ini_save(Efreet_Ini *ini, const char *file)
 {
+    char *dir;
     FILE *f;
     if (!ini || !ini->data) return 0;
 
+    dir = ecore_file_dir_get(file);
+    if (!ecore_file_mkpath(dir))
+    {
+        free(dir);
+        return 0;
+    }
+    free(dir);
     f = fopen(file, "wb");
     if (!f) return 0;
     eina_hash_foreach(ini->data, efreet_ini_section_save, f);
@@ -343,10 +342,10 @@ efreet_ini_section_add(Efreet_Ini *ini, const char *section)
     if (!ini || !section) return;
 
     if (!ini->data)
-      ini->data = eina_hash_string_small_new(EINA_FREE_CB(eina_hash_free));
+        ini->data = eina_hash_string_small_new(EINA_FREE_CB(eina_hash_free));
     if (eina_hash_find(ini->data, section)) return;
 
-    hash = eina_hash_string_small_new(free);
+    hash = eina_hash_string_small_new(EINA_FREE_CB(eina_stringshare_del));
     eina_hash_add(ini->data, section, hash);
 }
 
@@ -377,8 +376,8 @@ efreet_ini_string_set(Efreet_Ini *ini, const char *key, const char *value)
 {
     if (!ini || !key || !ini->section) return;
 
-    eina_hash_del(ini->section, key, NULL);
-    eina_hash_add(ini->section, key, strdup(value));
+    eina_hash_del_by_key(ini->section, key);
+    eina_hash_add(ini->section, key, eina_stringshare_add(value));
 }
 
 /**
@@ -614,7 +613,7 @@ efreet_ini_key_unset(Efreet_Ini *ini, const char *key)
 {
     if (!ini || !key || !ini->section) return;
 
-    eina_hash_del(ini->section, key, NULL);
+    eina_hash_del_by_key(ini->section, key);
 }
 
 /**
@@ -622,19 +621,19 @@ efreet_ini_key_unset(Efreet_Ini *ini, const char *key)
  * @return An allocated unescaped string
  * @brief Unescapes backslash escapes in a string
  */
-static char *
+static const char *
 efreet_ini_unescape(const char *str)
 {
     char *buf, *dest;
     const char *p;
 
     if (!str) return NULL;
-    if (!strchr(str, '\\')) return strdup(str);
-    buf = malloc(strlen(str) + 1);
+    if (!strchr(str, '\\')) return eina_stringshare_add(str);
+    buf = alloca(strlen(str) + 1);
 
     p = str;
     dest = buf;
-    while(*p)
+    while (*p)
     {
         if ((*p == '\\') && (p[1] != '\0'))
         {
@@ -668,7 +667,7 @@ efreet_ini_unescape(const char *str)
     }
 
     *(dest) = '\0';
-    return buf;
+    return eina_stringshare_add(buf);
 }
 
 static Eina_Bool
