@@ -32,6 +32,7 @@ void *alloca (size_t);
 #include <time.h>
 #include <fcntl.h>
 #include <fnmatch.h>
+#include <limits.h>
 
 #ifdef _WIN32
 # include <winsock2.h>
@@ -56,6 +57,17 @@ static Eina_Hash *mime_icons = NULL; /* contains cache with mime->icons */
 static Eina_Inlist *mime_icons_lru = NULL;
 static unsigned int _efreet_mime_init_count = 0;
 
+static const char *_mime_inode_symlink = NULL;
+static const char *_mime_inode_fifo = NULL;
+static const char *_mime_inode_chardevice = NULL;
+static const char *_mime_inode_blockdevice = NULL;
+static const char *_mime_inode_socket = NULL;
+static const char *_mime_inode_mountpoint = NULL;
+static const char *_mime_inode_directory = NULL;
+static const char *_mime_application_x_executable = NULL;
+static const char *_mime_application_octet_stream = NULL;
+static const char *_mime_text_plain = NULL;
+
 /**
  * @internal
  * @brief Holds whether we are big/little endian
@@ -218,8 +230,8 @@ efreet_mime_init(void)
 
     if (_efreet_mime_log_dom < 0) 
     {
-	ERROR("Efreet: Could not create a log domain for Efreet_mime.");
-	goto shutdown_efreet;
+        ERROR("Efreet: Could not create a log domain for Efreet_mime.");
+        goto shutdown_efreet;
     }
 
     efreet_mime_endianess = efreet_mime_endian_check();
@@ -233,13 +245,13 @@ efreet_mime_init(void)
 
     return _efreet_mime_init_count;
 
- unregister_log_domain:
+unregister_log_domain:
     eina_log_domain_unregister(_efreet_mime_log_dom);
- shutdown_efreet:
+shutdown_efreet:
     efreet_shutdown();
- shutdown_ecore_file:
+shutdown_ecore_file:
     ecore_file_shutdown();
- shutdown_ecore:
+shutdown_ecore:
     ecore_shutdown();
 
     return --_efreet_mime_init_count;
@@ -257,6 +269,17 @@ efreet_mime_shutdown(void)
 
     efreet_mime_icons_debug();
 
+    IF_RELEASE(_mime_inode_symlink);
+    IF_RELEASE(_mime_inode_fifo);
+    IF_RELEASE(_mime_inode_chardevice);
+    IF_RELEASE(_mime_inode_blockdevice);
+    IF_RELEASE(_mime_inode_socket);
+    IF_RELEASE(_mime_inode_mountpoint);
+    IF_RELEASE(_mime_inode_directory);
+    IF_RELEASE(_mime_application_x_executable);
+    IF_RELEASE(_mime_application_octet_stream);
+    IF_RELEASE(_mime_text_plain);
+
     IF_FREE_LIST(globs, efreet_mime_glob_free);
     IF_FREE_LIST(magics, efreet_mime_magic_free);
     IF_FREE_HASH(monitors);
@@ -651,12 +674,12 @@ efreet_mime_init_files(void)
      * We watch the directories so we can watch for new files
      */
     datadir = datahome;
-    snprintf(buf, PATH_MAX, "%s/mime", datadir);
+    snprintf(buf, sizeof(buf), "%s/mime", datadir);
     efreet_mime_monitor_add(buf);
 
     EINA_LIST_FOREACH(datadirs, l, datadir)
     {
-        snprintf(buf, PATH_MAX, "%s/mime", datadir);
+        snprintf(buf, sizeof(buf), "%s/mime", datadir);
         efreet_mime_monitor_add(buf);
     }
     efreet_mime_monitor_add("/etc/mime.types");
@@ -665,6 +688,17 @@ efreet_mime_init_files(void)
     efreet_mime_load_globs(datadirs, datahome);
     efreet_mime_load_magics(datadirs, datahome);
 
+    _mime_inode_symlink		   = eina_stringshare_add("inode/symlink");
+    _mime_inode_fifo		   = eina_stringshare_add("inode/fifo");
+    _mime_inode_chardevice	   = eina_stringshare_add("inode/chardevice");
+    _mime_inode_blockdevice	   = eina_stringshare_add("inode/blockdevice");
+    _mime_inode_socket		   = eina_stringshare_add("inode/socket");
+    _mime_inode_mountpoint	   = eina_stringshare_add("inode/mountpoint");
+    _mime_inode_directory	   = eina_stringshare_add("inode/directory");
+    _mime_application_x_executable = eina_stringshare_add("application/x-executable");
+    _mime_application_octet_stream = eina_stringshare_add("application/octet-stream");
+    _mime_text_plain               = eina_stringshare_add("text/plain");
+
     return 1;
 }
 
@@ -704,21 +738,21 @@ efreet_mime_special_check(const char *file)
 
 #ifndef _WIN32
         if (S_ISLNK(s.st_mode))
-            return "inode/symlink";
+        return _mime_inode_symlink;
 #endif
 
         if (S_ISFIFO(s.st_mode))
-            return "inode/fifo";
+            return _mime_inode_fifo;
 
         if (S_ISCHR(s.st_mode))
-            return "inode/chardevice";
+            return _mime_inode_chardevice;
 
         if (S_ISBLK(s.st_mode))
-            return "inode/blockdevice";
+            return _mime_inode_blockdevice;
 
 #ifndef _WIN32
         if (S_ISSOCK(s.st_mode))
-            return "inode/socket";
+            return _mime_inode_socket;
 #endif
 
         if (S_ISDIR(s.st_mode))
@@ -741,10 +775,10 @@ efreet_mime_special_check(const char *file)
             if (!lstat(parent, &s2))
             {
                 if (s.st_dev != s2.st_dev)
-                    return "inode/mount-point";
+                    return _mime_inode_mountpoint;
             }
 
-            return "inode/directory";
+            return _mime_inode_directory;
         }
 
         return NULL;
@@ -768,14 +802,14 @@ efreet_mime_fallback_check(const char *file)
     int i;
 
     if (ecore_file_can_exec(file))
-      return "application/x-executable";
+        return _mime_application_x_executable;
     
     if (!(f = fopen(file, "r"))) return NULL;
 
     i = fread(buf, 1, sizeof(buf), f);
     fclose(f);
 
-    if (i == 0) return "application/octet-stream";
+    if (i == 0) return _mime_application_octet_stream;
 
     /*
      * Check for ASCII control characters in the first 32 bytes.
@@ -788,10 +822,10 @@ efreet_mime_fallback_check(const char *file)
             (buf[i] != '\n') &&     /* Line Feed */
             (buf[i] != '\r') &&     /* Carriage Return */
             (buf[i] != '\t'))       /* Tab */
-            return "application/octet-stream";
+            return _mime_application_octet_stream;
     }
 
-    return "text/plain";
+    return _mime_text_plain;
 }
 
 /**
@@ -863,7 +897,7 @@ efreet_mime_mime_types_load(const char *file)
             strncpy(ext, pp, (p - pp));
             ext[p - pp] = 0;
 
-	    eina_hash_del(wild, ext, NULL);
+            eina_hash_del(wild, ext, NULL);
             eina_hash_add(wild, ext, (void*)eina_stringshare_add(mimetype));
         }
         while ((*p != '\n') && (*p != 0));
@@ -923,7 +957,7 @@ efreet_mime_shared_mimeinfo_globs_load(const char *file)
         {
             eina_hash_del(wild, &(ext[2]), NULL);
             eina_hash_add(wild, &(ext[2]),
-			  (void*)eina_stringshare_add(mimetype));
+                            (void*)eina_stringshare_add(mimetype));
         }
         else
         {
@@ -1102,7 +1136,7 @@ efreet_mime_shared_mimeinfo_magic_parse(char *data, int size)
                 entry->value = NULL;
 
                 mime->entries = eina_list_append(mime->entries, entry);
-           }
+            }
 
             switch(*ptr)
             {
@@ -1134,10 +1168,10 @@ efreet_mime_shared_mimeinfo_magic_parse(char *data, int size)
                 case '~':
                     ptr++;
                     entry->word_size = atoi(ptr);
-                    if ((entry->word_size != 0) && ((entry->word_size != 1)
-						     && (entry->word_size != 2)
-						     && (entry->word_size != 4)
-						     || (entry->value_len % entry->word_size)))
+                    if ((entry->word_size != 0) && (((entry->word_size != 1)
+                                    && (entry->word_size != 2)
+                                    && (entry->word_size != 4))
+                                || (entry->value_len % entry->word_size)))
                     {
                         /* Invalid, Destroy */
                         FREE(entry->value);
@@ -1582,7 +1616,7 @@ efreet_mime_icons_debug(void)
         }
 
         DBG("mime-icon entry: '%s' last used: %s",
-               entry->mime, ctime(&entry->timestamp));
+            entry->mime, ctime(&entry->timestamp));
 
         EINA_INLIST_FOREACH(entry->list, n)
             DBG("\tsize: %3u theme: '%s' icon: '%s'",
